package broker

import (
	"fmt"
	"log"

	"github.com/pivotal-cf/on-demand-service-broker-sdk/serviceadapter"
	"github.com/pivotal-cf/on-demand-service-broker/boshclient"
)

type deployer struct {
	boshClient      BoshClient
	instanceRepo    InstanceRepository
	serviceAdapter  ServiceAdapterClient
	serviceOffering ServiceCatalog
	serviceStemcell serviceadapter.Stemcell
	serviceReleases serviceadapter.ServiceReleases
	logger          *log.Logger
}

func NewDeployer(
	boshClient BoshClient,
	instanceRepo InstanceRepository,
	serviceAdapter ServiceAdapterClient,
	serviceOffering ServiceCatalog,
	serviceStemcell serviceadapter.Stemcell,
	serviceReleases serviceadapter.ServiceReleases,
	logger *log.Logger,
) Deployer {
	return deployer{
		boshClient:      boshClient,
		instanceRepo:    instanceRepo,
		serviceAdapter:  serviceAdapter,
		serviceOffering: serviceOffering,
		serviceStemcell: serviceStemcell,
		serviceReleases: serviceReleases,
		logger:          logger,
	}
}

func (d deployer) Deploy(instance Instance, planID, deploymentName string, operationType OperationType, arbitraryParams map[string]interface{}, oldManifest []byte) (Instance, error) {
	task, err := lastOperation(d.boshClient, deploymentName)
	if err != nil && err != noTasksError {
		return Instance{}, err
	}

	if err != noTasksError && (task.State == boshclient.Queued || task.State == boshclient.Processing) {
		d.logger.Printf("deployment %s is still in progress: %v", deploymentName, task)
		return Instance{}, DeploymentInProgressError{fmt.Errorf("deployment %s is still in progress", deploymentName)}
	}

	plan := d.serviceOffering.FindPlanById(planID)
	if plan == nil {
		return instance, fmt.Errorf("plan %s does not exist", planID)
	}

	serviceDeployment := serviceadapter.ServiceDeployment{DeploymentName: deploymentName, Releases: d.serviceReleases, Stemcell: d.serviceStemcell}

	d.logger.Printf("service adapter will generate manifest for instance %s\n", instance.ID)
	manifest, err := d.serviceAdapter.GenerateManifest(serviceDeployment, serviceAdapterPlan(*plan), arbitraryParams, oldManifest)
	if err != nil {
		return instance, fmt.Errorf("error generating manifest: %s", err)
	}

	taskID, err := d.boshClient.DeployRaw(manifest)
	if err != nil {
		return instance, fmt.Errorf("error deploying instance: %s\n", err)
	}

	d.logger.Printf("Bosh task id for %s instance %s was %d\n", operationType, instance.ID, taskID)
	instance.PlanID = planID
	instance.LastOperationType = operationType
	if err := d.instanceRepo.Save(instance); err != nil {
		return instance, fmt.Errorf("error saving instance: %s", err)
	}

	return instance, nil
}

func serviceAdapterPlan(plan Plan) serviceadapter.Plan {
	abridgedPlan := serviceadapter.Plan{
		Properties:     plan.Properties,
		InstanceGroups: plan.InstanceGroups,
	}
	return abridgedPlan
}
