package broker

import (
	"encoding/json"
	"fmt"

	"github.com/pivotal-cf/brokerapi"
)

func (b *Broker) Provision(instanceID string, details brokerapi.ProvisionDetails, asyncAllowed bool) (brokerapi.ProvisionedServiceSpec, error) {
	if !asyncAllowed {
		return brokerapi.ProvisionedServiceSpec{}, brokerapi.ErrAsyncRequired
	}

	arbitraryParams := map[string]interface{}{}
	if len(details.RawParameters) > 0 {
		if err := json.Unmarshal(details.RawParameters, &arbitraryParams); err != nil {
			return brokerapi.ProvisionedServiceSpec{}, brokerapi.ErrRawParamsInvalid
		}
	}

	isAsync, err := b.deployInstance(instanceID, details.PlanID, arbitraryParams, false)
	return brokerapi.ProvisionedServiceSpec{IsAsync: isAsync}, err
}

func (b *Broker) deployInstance(instanceID, planID string, arbitraryParams map[string]interface{}, replace bool) (bool, error) {
	errs := func(err error) (bool, error) {
		b.logger.Println(err)
		return true, err
	}

	planConfig := b.serviceOffering.FindPlanById(planID)
	if planConfig == nil {
		return errs(fmt.Errorf("plan %s not found", planID))
	}

	if !replace {
		exists, err := b.instanceRepo.InstanceExists(instanceID)
		if err != nil {
			return errs(fmt.Errorf("error fetching instance: %s", err))
		}
		if exists {
			return false, brokerapi.ErrInstanceAlreadyExists
		}

		count, err := b.instanceRepo.CountByPlanID(planConfig.ID)
		if err != nil {
			return errs(fmt.Errorf("error counting plans: %s", err))
		}

		if planConfig.Quotas != nil && count >= planConfig.Quotas.ServiceInstanceLimit {
			return errs(brokerapi.ErrPlanQuotaExceeded)
		}
	}

	_, err := b.deployer.Deploy(Instance{ID: instanceID, PlanID: planConfig.ID}, planID, deploymentName(instanceID), Create, arbitraryParams, nil)
	if err != nil {
		return errs(err)
	}

	return true, nil
}
