package broker_test

import (
	"errors"

	"github.com/pivotal-cf/on-demand-service-broker/broker"

	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
)

type ItRedeploysContext struct {
	instanceID    string
	arbParams     map[string]interface{}
	oldPlanID     string
	newPlanID     string
	redeployErr   error
	operationType broker.OperationType
}

func ItRedeploys(context *ItRedeploysContext) {
	var (
		instanceID     = context.instanceID
		deploymentName = broker.InstancePrefix + instanceID
		arbParams      = context.arbParams

		expectedPreviousManifest = []byte("old-manifest-fetched-from-bosh")
		oldPlanID                = context.oldPlanID
		taskID                   = 42
		newPlanID                = context.newPlanID
		existingInstanceInDB     broker.Instance
		operationType            = context.operationType
	)

	BeforeEach(func() {
		existingInstanceInDB = broker.Instance{ID: instanceID, PlanID: oldPlanID}
		instanceRepo.FindByIDReturns(existingInstanceInDB, nil)
	})

	Context("deployment goes well", func() {
		var manifest = []byte("updated-manifest")

		BeforeEach(func() {
			boshClient.DeployRawReturns(taskID, nil)
			instanceRepo.SaveReturns(nil)
			serviceAdapter.GenerateManifestReturns(manifest, nil)
			boshClient.GetManifestRawReturns(expectedPreviousManifest, nil)

			fakeDeployer.DeployReturns(broker.Instance{}, nil)
		})

		It("does not error", func() {
			Expect(context.redeployErr).NotTo(HaveOccurred())
		})

		It("fetches correct instance", func() {
			// fetches correct instance
			Expect(instanceRepo.FindByIDCallCount()).To(Equal(1))
			Expect(instanceRepo.FindByIDArgsForCall(0)).To(Equal(instanceID))
		})

		It("fetches correct manifest from bosh", func() {
			Expect(boshClient.GetManifestRawCallCount()).To(Equal(1))
			Expect(boshClient.GetManifestRawArgsForCall(0)).To(Equal(deploymentName))
		})

		It("deploys with the new planID", func() {
			Expect(fakeDeployer.DeployCallCount()).To(Equal(1))
			actualInstance, actualPlanID, actualDeploymentName, actualOperationType, actualArbitraryParams, actualOldManifest := fakeDeployer.DeployArgsForCall(0)
			Expect(actualInstance).To(Equal(existingInstanceInDB))
			Expect(actualPlanID).To(Equal(newPlanID))
			Expect(actualDeploymentName).To(Equal(broker.InstancePrefix + instanceID))
			Expect(actualArbitraryParams).To(Equal(arbParams))
			Expect(actualOldManifest).To(Equal(expectedPreviousManifest))
			Expect(actualOperationType).To(Equal(operationType))
		})

		Context("when the plan quota has been exceeded", func() {
			BeforeEach(func() {
				instanceRepo.CountByPlanIDReturns(serviceInstanceLimit+1, nil)
			})

			It("updates anyway", func() {
				Expect(fakeDeployer.DeployCallCount()).To(Equal(1))
			})
		})
	})

	Context("when bosh fails to fetch previous manifest", func() {
		BeforeEach(func() {
			boshClient.GetManifestRawReturns(nil, errors.New("no manifest for you"))
		})

		It("wraps the error", func() {
			Expect(context.redeployErr).To(MatchError(ContainSubstring("no manifest for you")))
		})
	})

	Context("when find instance by id fails", func() {
		BeforeEach(func() {
			boshClient.GetManifestRawReturns(expectedPreviousManifest, nil)
			instanceRepo.FindByIDReturns(broker.Instance{}, errors.New("no instance for you"))
		})

		It("wraps the error", func() {
			Expect(context.redeployErr).To(MatchError(ContainSubstring("no instance for you")))
		})
	})

}
